/**************************************************************************
*  Copyright (c) 2017-2018 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  09.01.2017  mifi  First Version.
**************************************************************************/
#define __MAIN_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <windows.h>
#include <stdio.h>
#include <time.h>

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

#define VERSION         "1.06"
#define MAX_INPUT_SIZE  (4*1024*1024)

#define BYTES_IN_LINE   12

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static char szInFile[_MAX_PATH];    /* Input file name */
static char szOutFile[_MAX_PATH];   /* Output file name */
static char szVariable[_MAX_PATH];  /* Variable name */

static char szOutFileHeader[_MAX_PATH];   /* Header file */
static char szOutFileSource[_MAX_PATH];   /* Source file */

static char szUpperName[_MAX_PATH];

static BYTE   InImage[MAX_INPUT_SIZE];

static DWORD dNullTerminated = 0;

static const char Header1[] = 
"/**************************************************************************\r\n"
"*  This file was created automatically by xbin2c.exe (www.emb4fun.de).\r\n"
"*\r\n"
"*  Source: %s\r\n"
"**************************************************************************/\r\n"
"#if !defined(__%s_H__)\r\n"
"#define __%s_H__\r\n"
"\r\n"
"/**************************************************************************\r\n"
"*  Extern Definitions\r\n"
"**************************************************************************/\r\n"
"extern const unsigned char %s[%d + %d];\r\n"
"\r\n"
"#endif /* __%s_H__ */\r\n"
"/*** EOF ***/\r\n";

static const char Header2[] = 
"/**************************************************************************\r\n"
"*  This file was created automatically by xbin2c.exe (www.emb4fun.de).\r\n"
"*\r\n"
"*  Source: %s\r\n"
"**************************************************************************/\r\n"
"#define __%s_C__\r\n"
"\r\n"
"/**************************************************************************\r\n"
"*  Global Data\r\n"
"**************************************************************************/\r\n"
"const unsigned char %s[%d + %d] __attribute__((aligned(4))) =\r\n";

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  OutputStartMessage                                                   */
/*                                                                       */
/*  Output start message.                                                */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void OutputStartMessage (void)
{
   printf("\n");   
   printf("xbin2c v%s compiled "__DATE__" "__TIME__"\n", VERSION);
   printf("(c) 2017 by Michael Fischer (www.emb4fun.de)\n");
   printf("\n");
} /* OutputStartMessage */

/*************************************************************************/
/*  OutputUsage                                                          */
/*                                                                       */
/*  Output "usage" message.                                              */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void OutputUsage (void)
{
  printf("Usage: xbin2c -i:Infile -o[n]:Outfile -v:Variable\n");
  printf("\n");
  printf("  -i   Input file, e.g. -i:test.bin\n");
  printf("  -o   Output file without .c or .h, e.g. -o:test\n");
  printf("  -on  Like -o but null terminated\n");
  printf("  -v   Variable, e.g. -v:_test_data\n");
} /* OutputUsage */

/*************************************************************************/
/*  CreateOutputImage                                                    */
/*                                                                       */
/*  Create the output image file.                                        */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void CreateOutputImage (void)
{
   FILE  *hInFile;
   DWORD  dInFileSize;
   FILE  *hOutFile;
   DWORD  dIndex;
   DWORD  dLineCount;
   DWORD  dRestCount;
   DWORD  dAddress;
   int     x;

   /* Create header and source filename */
   _snprintf(szOutFileHeader, sizeof(szOutFileHeader), "%s.h", szOutFile);
   _snprintf(szOutFileSource, sizeof(szOutFileSource), "%s.c", szOutFile);

   /* Create upper NAME for "#if !defined(__NAME__H__)" */
   _snprintf(szUpperName,  sizeof(szUpperName), "%s", szOutFile);
   for(dIndex=0; dIndex<strlen(szUpperName); dIndex++)
   {
      szUpperName[dIndex] = toupper(szUpperName[dIndex]);
   }

   /* Check if input file is available */
   hInFile = fopen(szInFile, "rb");
   if (NULL == hInFile)
   {
      printf("Error: Input file <%s> not found\n", szInFile);
      exit(1);
   }

   /* Get size of input file */
   fseek(hInFile, 0, SEEK_END);
   dInFileSize = ftell(hInFile);
   fseek(hInFile, 0, SEEK_SET);
   
   /* Read input file data */
   if (dInFileSize > MAX_INPUT_SIZE)
   {
      printf("Error: Input file size > %d\n", MAX_INPUT_SIZE);
      fclose(hInFile);
      exit(1);
   }

   /* "Clear" InImage */
   memset(InImage, 0x00, sizeof(InImage));
   
   /* Read input file */
   fread(InImage, 1, dInFileSize, hInFile); 
   
   /* Close input file */
   fclose(hInFile);
   
   /*
    * Create "header" file
    */
   hOutFile = fopen(szOutFileHeader, "wb");
   if (NULL == hOutFile)
   {
      /* File could not created */
      printf("Error: Could not create \"%s\"\n", szOutFileHeader);
      exit(1);
   }
   
   fprintf(hOutFile, Header1, 
           szInFile, szUpperName, szUpperName, szVariable, dInFileSize, dNullTerminated, szUpperName);
   
   fclose(hOutFile);

   
   /*
    * Create "source" file
    */
   hOutFile = fopen(szOutFileSource, "wb");
   if (NULL == hOutFile)
   {
      /* File could not created */
      printf("Error: Could not create \"%s\"\n", szOutFileSource);
      exit(1);
   }
   
   fprintf(hOutFile, Header2, 
           szInFile, szUpperName, szVariable, dInFileSize, dNullTerminated);
           
   fprintf(hOutFile, "{\r\n");

   /* "Null terminated" option */
   dInFileSize += dNullTerminated;
   
   dLineCount = dInFileSize / BYTES_IN_LINE;
   dRestCount = dInFileSize % BYTES_IN_LINE;

   dAddress = 0;
   if (dLineCount)
   {
      for(dIndex=0; dIndex<dLineCount; dIndex++)
      {
         fprintf(hOutFile, "   ");
         for(x=0; x<BYTES_IN_LINE; x++)
         {
            fprintf(hOutFile, "0x%02x", InImage[dAddress++]);
            if (dAddress != dInFileSize)
            {
               fprintf(hOutFile, ", ");
            }
         }
         fprintf(hOutFile, "\r\n");
      }
   }
   
   if (dRestCount)
   {
      fprintf(hOutFile, "   ");
      for(dIndex=0; dIndex<dRestCount; dIndex++)
      {
         fprintf(hOutFile, "0x%02x", InImage[dAddress++]);
         if (dAddress != dInFileSize)
         {
            fprintf(hOutFile, ", ");
         }
      }
      fprintf(hOutFile, "\r\n");
   }      
   
   fprintf(hOutFile, "};\r\n");
   fprintf(hOutFile, "\r\n");
   fprintf(hOutFile, "/*** EOF ***/\r\n");
   
   fclose(hOutFile);

} /* CreateOutputImage */ 

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  main                                                                 */
/*                                                                       */
/*  In    : argc, argv                                                   */
/*  Out   : none                                                         */
/*  Return: 0 / error cause                                              */
/*************************************************************************/
int main (int argc, char **argv)
{
   int   nIndex;
   int   nLen;
   char *pString;
   
   szInFile[0]   = 0;
   szOutFile[0]  = 0;     
   szVariable[0] = 0;
   
   OutputStartMessage();
   
   if (argc != 4)
   {
      OutputUsage();
   }
   else
   {
      /* 
       * Parse command line 
       */
      for (nIndex=0; nIndex<argc; nIndex++)
      {
         /* Input file */
         if ((pString = strstr(argv[nIndex], "-i:")) != NULL)
         {
            pString += 3;  /* Jump over "-i:" */
            nLen     = strlen(pString);
            if (nLen < (sizeof(szInFile)-1))
            {
               strcpy(szInFile, pString);
            }
            else
            {
               printf("Error: Input name len > %d\n", (sizeof(szInFile)-1));
               exit(1);
            }
         } /* Parse input file name */
         
         /* Output file -o */
         if ((pString = strstr(argv[nIndex], "-o:")) != NULL)
         {
            pString += 3;  /* Jump over "-o:" */
            nLen     = strlen(pString);
            if (nLen < (sizeof(szOutFile)-1))
            {
               strcpy(szOutFile, pString);
            }
            else
            {
               printf("Error: Output name len > %d\n", (sizeof(szOutFile)-1));
               exit(1);
            }
         } /* Parse output file name */

         /* Output file -on */
         if ((pString = strstr(argv[nIndex], "-on:")) != NULL)
         {
            pString += 4;  /* Jump over "-n:" */
            nLen     = strlen(pString);
            if (nLen < (sizeof(szOutFile)-1))
            {
               strcpy(szOutFile, pString);
               dNullTerminated = 1;
            }
            else
            {
               printf("Error: Output name len > %d\n", (sizeof(szOutFile)-1));
               exit(1);
            }
         } /* Parse output file name */
         
         /* Variable name */
         if ((pString = strstr(argv[nIndex], "-v:")) != NULL)
         {
            pString += 3;  /* Jump over "-v:" */
            nLen     = strlen(pString);
            if (nLen < (sizeof(szVariable)-1))
            {
               strcpy(szVariable, pString);
            }
            else
            {
               printf("Error: Variable name len > %d\n", (sizeof(szVariable)-1));
               exit(1);
            }
         } /* Parse output file name */
         
      } /* end for loop */
      
      if (0 == szInFile[0])
      {
         printf("Error: No input file specified\n");
         exit(1);
      }
      
      if (0 == szOutFile[0])
      {
         printf("Error: No Output file specified\n");
         exit(1);
      }
      
      if (0 == szVariable[0])
      {
         printf("Error: No Variable specified\n");
         exit(1);
      }
      
      /*
       * Here the command line was parsed and no error was available
       */
      printf("Input file : %s\n", szInFile);
      printf("Output file: %s.c, %s.h\n", szOutFile, szOutFile);
      printf("Variable   : %s\n", szVariable);
      
      CreateOutputImage();
   }      
   
   return(0);
} /* main */

/*** EOF ***/
